/*
The software in this package is distributed under the GNU General
Public License version 2 (with a special exception described below).

A copy of GNU General Public License (GPL) is included in this distribution,
in the file COPYING.GPL.

As a special exception, if other files instantiate templates or use macros
or inline functions from this file, or you compile this file and link it
with other works to produce a work based on this file, this file
does not by itself cause the resulting work to be covered
by the GNU General Public License.

However the source code for this file must still be made available
in accordance with section (3) of the GNU General Public License.

This exception does not invalidate any other reasons why a work based
on this file might be covered by the GNU General Public License.
*/
/** @file
 *
 * tests for the module tmpfstream
 *
 * (c) Copyright 2010 by Intra2net AG
 */

//#define NOISEDEBUG

#define BOOST_TEST_DYN_LINK
#include <boost/test/unit_test.hpp>

#include <iostream>
#include <filefunc.hxx>
#include <tmpfstream.hpp>

#ifdef NOISEDEBUG
#define DOUT(msg) std::cout << msg << std::endl
#else
#define DOUT(msg) do {} while (0)
#endif

using namespace std;
using namespace I2n;

namespace bio = boost::iostreams;

BOOST_AUTO_TEST_SUITE(TestTmpfstream)

BOOST_AUTO_TEST_CASE(Tmpofstream)
{
    tmpofstream tmpf("./tmp.XXXXXX");

    BOOST_CHECK_EQUAL( true, tmpf.is_open() );
    BOOST_CHECK_EQUAL( false, tmpf.get_tmp_filename().empty() );

    Stat stat(tmpf.get_tmp_filename());

    DOUT("filename : " << tmpf.get_tmp_filename());

    BOOST_CHECK_EQUAL( true, (bool)stat );
    BOOST_CHECK_EQUAL( true, stat.is_regular_file() );

    tmpf << "hello world" << endl;

    tmpf.close();
    BOOST_CHECK_EQUAL( false, tmpf.is_open() );

    Stat stat2(tmpf.get_tmp_filename());

    DOUT("stated path: " << stat2.path());
    DOUT("filesize: " << stat2.size());

    BOOST_CHECK_EQUAL( true, stat2.size() > 0 );

    BOOST_CHECK_EQUAL( true, tmpf.unlink() );

    Stat stat3(tmpf.get_tmp_filename());
    BOOST_CHECK_EQUAL( false, (bool)stat3 );

} 

BOOST_AUTO_TEST_CASE(TmpofstreamUnlink)
{
    tmpofstream tmpf("./tmp.XXXXXX");

    BOOST_CHECK_EQUAL( true, tmpf.is_open() );

    Stat stat(tmpf.get_tmp_filename());
    BOOST_CHECK_EQUAL( true, (bool)stat );
    BOOST_CHECK_EQUAL( true, stat.is_regular_file() );

    tmpf << "hello world" << endl;

    // keep open but delete file
    tmpf.unlink();
    stat.recheck();
    BOOST_CHECK_EQUAL( false, (bool)stat );

    // writing to already deleted file
    // POSIX explicitly allows this
    tmpf << "hello world" << endl;
    tmpf.flush();
    BOOST_CHECK_EQUAL( true, tmpf.good() );

    // file still gone after closing
    tmpf.close();
    stat.recheck();
    BOOST_CHECK_EQUAL( false, (bool)stat );
} 

BOOST_AUTO_TEST_CASE(TmpofstreamMove1)
{
    // just to be sure
    unlink(".foobar");

    tmpofstream tmpf("./tmp.XXXXXX");

    BOOST_CHECK_EQUAL( true, tmpf.is_open() );

    Stat stat(tmpf.get_tmp_filename());
    BOOST_CHECK_EQUAL( true, (bool)stat );
    BOOST_CHECK_EQUAL( true, stat.is_regular_file() );

    tmpf << "hello world" << endl;
    BOOST_CHECK_EQUAL( true, tmpf.sync() );

    BOOST_CHECK_EQUAL( true, tmpf.move(".foobar",false) );

    stat.recheck();
    BOOST_CHECK_EQUAL( false, (bool)stat );

    Stat stat2(".foobar");
    BOOST_CHECK_EQUAL( true, (bool)stat2 );

    tmpf.close();

    stat.recheck();
    BOOST_CHECK_EQUAL( false, (bool)stat );

    stat2.recheck();
    BOOST_CHECK_EQUAL( true, (bool)stat2 );

    unlink(".foobar");
} 

BOOST_AUTO_TEST_CASE(TmpofstreamMove2)
{
    // just to be sure
    unlink(".foobar");

    tmpofstream tmpf("./tmp.XXXXXX");

    BOOST_CHECK_EQUAL( true, tmpf.is_open() );

    Stat stat(tmpf.get_tmp_filename());
    BOOST_CHECK_EQUAL( true, (bool)stat );
    BOOST_CHECK_EQUAL( true, stat.is_regular_file() );

    tmpf << "hello world" << endl;
    tmpf.flush();

    BOOST_CHECK_EQUAL( true, tmpf.move(".foobar",true) );

    stat.recheck();
    BOOST_CHECK_EQUAL( false, (bool)stat );

    Stat stat2(".foobar");
    BOOST_CHECK_EQUAL( true, (bool)stat2 );

    tmpf.close();

    stat.recheck();
    BOOST_CHECK_EQUAL( false, (bool)stat );

    stat2.recheck();
    BOOST_CHECK_EQUAL( true, (bool)stat2 );

    unlink(".foobar");
} 

BOOST_AUTO_TEST_CASE(TmpofstreamMoveOverwrite)
{
    // prepare other file
    write_file(".foobar","blah");

    tmpofstream tmpf("./tmp.XXXXXX");

    BOOST_CHECK_EQUAL( true, tmpf.is_open() );

    Stat stat(tmpf.get_tmp_filename());
    BOOST_CHECK_EQUAL( true, (bool)stat );
    BOOST_CHECK_EQUAL( true, stat.is_regular_file() );

    tmpf << "hello world" << endl;
    tmpf.flush();

    // no overwrite fails
    BOOST_CHECK_EQUAL( false, tmpf.move(".foobar",false) );

    // overwrite succeeds
    BOOST_CHECK_EQUAL( true, tmpf.move(".foobar",true) );

    stat.recheck();
    BOOST_CHECK_EQUAL( false, (bool)stat );

    Stat stat2(".foobar");
    BOOST_CHECK_EQUAL( true, (bool)stat2 );

    tmpf.close();

    unlink(".foobar");
} 

BOOST_AUTO_TEST_CASE(TmpofstreamMoveAfterUnlink)
{
    // just to be sure
    unlink(".foobar");

    tmpofstream tmpf("./tmp.XXXXXX");

    BOOST_CHECK_EQUAL( true, tmpf.is_open() );

    Stat stat(tmpf.get_tmp_filename());
    BOOST_CHECK_EQUAL( true, (bool)stat );
    BOOST_CHECK_EQUAL( true, stat.is_regular_file() );

    tmpf << "hello world" << endl;
    tmpf.flush();

    BOOST_CHECK_EQUAL( true, tmpf.unlink() );

    BOOST_CHECK_EQUAL( true, tmpf.is_unlinked() );

    BOOST_CHECK_EQUAL( false, tmpf.move(".foobar") );

    unlink(".foobar");
} 

BOOST_AUTO_TEST_CASE(TmpofstreamDoubleMove)
{
    // just to be sure
    unlink(".foobar");
    unlink(".foobar2");

    tmpofstream tmpf("./tmp.XXXXXX");

    BOOST_CHECK_EQUAL( true, tmpf.is_open() );

    Stat stat(tmpf.get_tmp_filename());
    BOOST_CHECK_EQUAL( true, (bool)stat );

    tmpf << "hello world" << endl;
    tmpf.flush();

    BOOST_CHECK_EQUAL( true, tmpf.move(".foobar") );

    stat.recheck();
    BOOST_CHECK_EQUAL( false, (bool)stat );

    Stat stat2(".foobar");
    BOOST_CHECK_EQUAL( true, (bool)stat2 );

    BOOST_CHECK_EQUAL( true, tmpf.move(".foobar2") );

    stat.recheck();
    BOOST_CHECK_EQUAL( false, (bool)stat );

    stat2.recheck();
    BOOST_CHECK_EQUAL( false, (bool)stat2 );

    Stat stat3(".foobar2");
    BOOST_CHECK_EQUAL( true, (bool)stat3 );

    tmpf.unlink();
} 

BOOST_AUTO_TEST_CASE(TmpfstreamReadWrite)
{
    tmpfstream tmpf("./tmp.XXXXXX",std::ios_base::out | std::ios_base::in );
    BOOST_CHECK_EQUAL( true, tmpf.is_open() );

    tmpf << "hello world" << endl;

    tmpf.seekg(0);
    string input;
    getline(tmpf,input);

    BOOST_CHECK_EQUAL( "hello world", input );

    tmpf.unlink();
    tmpf.close();
}

BOOST_AUTO_TEST_CASE(Tmpfcopystream)
{
    write_file(".foobar","blah");

    tmpfcopystream tmpf(".foobar");
    BOOST_CHECK_EQUAL( true, tmpf.is_open() );

    tmpf << "hello world";
    tmpf.flush();

    BOOST_CHECK_EQUAL( "blah", read_file(".foobar") );

    tmpf.close();
    BOOST_CHECK_EQUAL( false, tmpf.is_open() );

    BOOST_CHECK_EQUAL( "hello world", read_file(".foobar") );

    unlink(".foobar");
}

BOOST_AUTO_TEST_CASE(TmpfcopystreamDestructor)
{
    write_file(".foobar","blah");

    {
        tmpfcopystream tmpf(".foobar");
        BOOST_CHECK_EQUAL( true, tmpf.is_open() );

        tmpf << "hello world";
        tmpf.flush();

        BOOST_CHECK_EQUAL( "blah", read_file(".foobar") );
    }

    BOOST_CHECK_EQUAL( "hello world", read_file(".foobar") );

    unlink(".foobar");
}

BOOST_AUTO_TEST_CASE(TmpfcopystreamAbort)
{
    write_file(".foobar","blah");

    tmpfcopystream tmpf(".foobar");
    BOOST_CHECK_EQUAL( true, tmpf.is_open() );

    tmpf << "hello world";
    tmpf.flush();

    // abort the overwriting, keep the old file
    tmpf.unlink();

    BOOST_CHECK_EQUAL( "blah", read_file(".foobar") );

    unlink(".foobar");
}

BOOST_AUTO_TEST_CASE(TmpfcopystreamChmod)
{
    write_file(".foobar","blah");
    I2n::chmod(".foobar",0644);
    Stat stat_orig(".foobar");
    BOOST_CHECK_EQUAL( 0644, stat_orig.mode() );

    tmpfcopystream tmpf(".foobar");

    tmpf << "hello world";
    tmpf.flush();

    tmpf.set_filemode_on_close(0640);

    Stat stat_tmp(tmpf.get_tmp_filename());
    BOOST_CHECK_EQUAL( 0600, stat_tmp.mode() );

    tmpf.close();

    stat_orig.recheck();
    BOOST_CHECK_EQUAL( 0640, stat_orig.mode() );

    unlink(".foobar");
}

BOOST_AUTO_TEST_CASE(CreateError)
{
    tmpofstream tmpf("/this/dir/really/does/not/exist/tmp.XXXXXX");

    BOOST_CHECK_EQUAL( false, tmpf.is_open() );
}

BOOST_AUTO_TEST_SUITE_END()
